@@ -3,10 +3,16 @@
 See http://www-huber.embl.de/users/anders/HTSeq for documentation.
 """
 
-import itertools, warnings
-
-from _HTSeq import *
+import itertools, warnings, os
 
+try:
+   from _HTSeq import *
+except ImportError:
+   if os.path.isfile( "setup.py" ):
+      raise ImportError( "Cannot import 'HTSeq' when working directory is HTSeq's own build directory.")
+   else:
+      raise
+      
 from _version import __version__
 
 #from vcf_reader import *
@@ -161,7 +167,7 @@ def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
       attrStr = attrStr[:-1]
    d = {}
    first_val = "_unnamed_"
-   for (i, attr) in itertools.izip( itertools.count(), attrStr.split( ";" ) ):
+   for (i, attr) in itertools.izip( itertools.count(), _HTSeq.quotesafe_split( attrStr ) ):
       if _re_attr_empty.match( attr ):
          continue
       if attr.count( '"' ) not in ( 0, 2 ):
@@ -194,7 +200,7 @@ class GFF_Reader( FileOrSequence ):
    Iterating over the object then yields GenomicFeature objects.
    """
    
-   def __init__( self, filename_or_sequence, end_included=False ):
+   def __init__( self, filename_or_sequence, end_included=True ):
       FileOrSequence.__init__( self, filename_or_sequence )
       self.end_included = end_included
       self.metadata = {}
@@ -214,9 +220,9 @@ class GFF_Reader( FileOrSequence ):
             strand, frame, attributeStr ) = line.split( "\t", 8 )   
          ( attr, name ) = parse_GFF_attribute_string( attributeStr, True )
          if self.end_included:
-            iv = GenomicInterval( seqname, int(start)-1, int(end)-1, strand )
-         else:
             iv = GenomicInterval( seqname, int(start)-1, int(end), strand )
+         else:
+            iv = GenomicInterval( seqname, int(start)-1, int(end)-1, strand )
          f = GenomicFeature( name, feature, iv )
          if score != ".":
             score = float( score )
@@ -652,6 +658,7 @@ class VariantCall( object ):
         self.qual   = qual
         self.filter = filtr
         self.info   = info
+        self._original_line = None
     
     @classmethod
     def fromdict( cls, dictionary ):
@@ -664,7 +671,8 @@ class VariantCall( object ):
         ret.qual    = dictionary["qual"]
         ret.filter  = dictionary["filter"]
         ret.info    = dictionary["info"]
-    
+        ret._original_line = None
+
     @classmethod
     def fromline( cls, line, nsamples = 0, sampleids = [] ):
         ret = cls()
@@ -682,6 +690,7 @@ class VariantCall( object ):
                 spos += 1
         ret.pos = GenomicPosition( ret.chrom, int(ret.pos) )
         ret.alt = ret.alt.split(",")
+        ret._original_line = line
         return ret
     
     def infoline( self ):
@@ -690,8 +699,29 @@ class VariantCall( object ):
         else:
             return self.info
     
+    def get_original_line( self ):
+       warnings.warn( "Original line is empty, probably this object was created from scratch and not from a line in a .vcf file!" )
+       return self._original_line
+    
+    def sampleline( self ):
+       if self.format == None:
+          print >> sys.stderr, "No samples in this variant call!" 
+          return ""
+       keys = self.format
+       ret = [ ":".join( keys ) ]
+       for sid in self.samples:
+          tmp = []
+          for k in keys:
+             if k in self.samples[sid]:
+                tmp.append( self.samples[sid][k] )
+          ret.append( ":".join(tmp) )
+       return "\t".join( ret )
+    
     def to_line( self ):
-        return "\t".join( map( str, [ self.chrom, self.pos, self.id, self.ref, self.alt, self.qual, self.filter, self.infoline ] ) )
+       if self.format == None:
+          return "\t".join( map( str, [ self.pos.chrom, self.pos.pos, self.id, self.ref, ",".join( self.alt ), self.qual, self.filter, self.infoline() ] ) ) + "\n"
+       else:
+          return "\t".join( map( str, [ self.pos.chrom, self.pos.pos, self.id, self.ref, ",".join( self.alt ), self.qual, self.filter, self.infoline(), self.sampleline() ] ) ) + "\n"
     
     def __descr__( self ):
         return "<VariantCall at %s, ref '%s', alt %s >" % (str(self.pos).rstrip("/."), self.ref, str(self.alt).strip("[]"))
@@ -768,6 +798,20 @@ class VCF_Reader( FileOrSequence ):
             else:
                 break
     
+    def meta_info( self, header_filename = None ):
+       ret = []
+       if header_filename == None:
+          the_iter = FileOrSequence.__iter__( self )
+       else:
+          the_iter = open( header_filename, "r" )
+       
+       for line in the_iter:
+          if line.startswith( '#' ):
+             ret.append( line )
+          else:
+             break
+       return ret
+    
     def __iter__( self ):
         for line in FileOrSequence.__iter__( self ):
             if line == "\n" or line.startswith( '#' ):
@@ -785,7 +829,7 @@ class BAM_Reader( object ):
         try:
            import pysam
         except ImportError:
-           print "Please Install PySam to use the BAM_Reader Class (http://code.google.com/p/pysam/)"
+           sys.stderr.write( "Please Install PySam to use the BAM_Reader Class (http://code.google.com/p/pysam/)" )
            raise
     
     def __iter__( self ):
@@ -795,6 +839,22 @@ class BAM_Reader( object ):
             yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
             self.record_no += 1
     
+    def fetch( self, reference = None, start = None, end = None, region = None ):
+        sf = pysam.Samfile(self.filename, "rb")
+        self.record_no = 0
+        try:
+           for pa in sf.fetch( reference, start, end, region ):
+              yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
+              self.record_no += 1
+        except ValueError as e:
+           if e.message == "fetch called on bamfile without index":
+              print "Error: ", e.message
+              print "Your bam index file is missing or wrongly named, convention is that file 'x.bam' has index file 'x.bam.bai'!"
+           else:
+              raise
+        except:
+           raise
+
     def get_line_number_string( self ):
         if self.record_no == -1:
             return "unopened file %s" % ( self.filename )
@@ -811,4 +871,33 @@ class BAM_Reader( object ):
         for pa in self.sf.fetch( iv.chrom, iv.start+1, iv.end ):
             yield SAM_Alignment.from_pysam_AlignedRead( pa, self.sf )
     
+    def get_header_dict( self ):
+       sf = pysam.Samfile(self.filename, "rb")
+       return sf.header
+    
                
+class BAM_Writer( object ):
+   def __init__( self, filename, template = None, referencenames = None, referencelengths = None, text = None, header = None ):
+      try:
+         import pysam
+      except ImportError:
+         sys.stderr.write( "Please Install PySam to use the BAM_Writer Class (http://code.google.com/p/pysam/)" )
+         raise
+      
+      self.filename = filename
+      self.template = template
+      self.referencenames = referencenames
+      self.referencelengths = referencelengths
+      self.text = text
+      self.header = header
+      self.sf = pysam.Samfile( self.filename, mode="wb", template = self.template, referencenames = self.referencenames, referencelengths = self.referencelengths, text = self.text, header = self.header )
+      
+   @classmethod
+   def from_BAM_Reader( cls, fn, br ):
+      return BAM_Writer( filename = fn, header = br.get_header_dict() )
+   
+   def write( self, alnmt):
+      self.sf.write( alnmt.to_pysam_AlignedRead( self.sf ) )
+   
+   def close( self ):
+      self.sf.close()
